@@ -86,6 +86,15 @@ Defaults to the version currently running (%s)." % LOCAL_MACOS_VERSION))
     help=("Don't trim margins from the mask. By default, transparent margins are trimmed from all 4 sides.")
   )
 
+  parser.add_argument(
+    "--set-icon-using",
+    type=str,
+    metavar="TOOL",
+    default="auto",
+    help="Tool to used to set the icon of the target: auto (default), seticon, Rez.\n\
+Rez usually produces a smaller \"resource fork\" for the icon, but only works if \
+XCode command line tools are already installed and if you're using a folder target.")
+
   parser.add_argument(
     "--verbose", "-v",
     action="store_true",
@@ -124,6 +133,19 @@ Defaults to the version currently running (%s)." % LOCAL_MACOS_VERSION))
     sys.stderr.write("Invalid color scheme. Defaulting to light.\n")
     effective_color_scheme = "light"
 
+  set_icon_using = args.set_icon_using
+  if set_icon_using == "auto":
+    # `seticon` is the most compatible at the moment
+    set_icon_using = "seticon"
+  if set_icon_using == "rez":
+    # Accept lowercase. The actual binary is `Rez` (`man Rez` works but `man
+    # rez` doesn't), but macOS is case-insensitive and `rez` matches the case
+    # you'd expect if there weren't legacy considerations.
+    set_icon_using = "Rez"
+  if set_icon_using not in ["seticon", "Rez"]:
+    sys.stderr.write("Invalid icon tool specified. Defaulting to seticon.\n")
+    set_icon_using = "seticon"
+
   if args.macOS in ["10.5", "10.6", "10.7", "10.8", "10.9"]:
     # http://arstechnica.com/apple/2007/10/mac-os-x-10-5/4/
     folder_style = "pre-Yosemite"
@@ -138,13 +160,15 @@ Defaults to the version currently running (%s)." % LOCAL_MACOS_VERSION))
     if effective_color_scheme == "dark":
       folder_style = "BigSur.dark"
 
-  print("Using folder style:", folder_style)
-
   template_folder = os.path.join(
     data_folder, "GenericFolderIcon.%s.iconset" % folder_style)
 
   convert_path = "convert"
   iconutil_path = "iconutil"
+  sips_path = "sips"
+  DeRez_path = "DeRez"
+  Rez_path = "Rez"
+  SetFile_path = "SetFile"
   seticon_path = os.path.join(data_folder, "lib", "seticon")
 
   ################################################################
@@ -366,6 +390,7 @@ or
     else:
       temp_folder = tempfile.mkdtemp()
 
+    original_target = target
     if target:
       iconset_folder = os.path.join(temp_folder, "iconset.iconset")
       icns_file = os.path.join(temp_folder, "icns.icns")
@@ -404,7 +429,7 @@ or
           "fill": "rgb(8, 134, 206)"
         },
         "dimensions": [
-          ["16x16",      16, (768, 384), (12, 6, 2), (0, 2), (2, 0, "0.5")],
+          ["16x16",      16, (768, 384), (12, 6, 2), (0, 2), (1, 0, "0.5")],
           ["16x16@2x",   32, (768, 384), (24, 12, 2), (0, 2), (2, 1, "0.35")],
           ["32x32",      32, (768, 384), (24, 12, 2), (0, 2), (2, 1, "0.35")], # Can be excluded
           ["32x32@2x",   64, (768, 384), (48, 24, 3), (0, 2), (2, 1, "0.6")],
@@ -421,7 +446,7 @@ or
           "fill": "rgb(6, 111, 194)"
         },
         "dimensions": [
-          ["16x16",      16, (768, 384), (12, 6, 2), (0, 2), (2, 0, "0.5")],
+          ["16x16",      16, (768, 384), (12, 6, 2), (0, 2), (1, 0, "0.5")],
           ["16x16@2x",   32, (768, 384), (24, 12, 2), (0, 2), (2, 1, "0.35")],
           ["32x32",      32, (768, 384), (24, 12, 2), (0, 2), (2, 1, "0.35")], # Can be excluded
           ["32x32@2x",   64, (768, 384), (48, 24, 3), (0, 2), (2, 1, "0.6")],
@@ -457,6 +482,7 @@ or
       }
     }
 
+    print("[%s] Using folder style: %s" % (print_prefix, folder_style))
     f = functools.partial(create_iconset, folder_style, print_prefix,
                     mask, temp_folder, iconset_folder, inputs[folder_style]["colors"])
     processes = map(f, inputs[folder_style]["dimensions"])
@@ -474,21 +500,80 @@ or
       iconset_folder
     ])
 
-    # Make sure seticon is executable.
-    subprocess.check_call([
-      "chmod",
-      "+x",
-      seticon_path
-    ])
+    if set_icon_using == "seticon":
+      # Make sure seticon is executable.
+      subprocess.check_call([
+        "chmod",
+        "+x",
+        seticon_path
+      ])
 
-    if args.verbose:
-      print("[%s] Setting icon for target." % (print_prefix))
-    # Set icon for target.
-    subprocess.check_call([
-      seticon_path,
-      icns_file,
-      target
-    ])
+      if args.verbose:
+        print("[%s] Setting icon for target %s with seticon." % (print_prefix, target))
+      # Set icon for target.
+      subprocess.check_call([
+        seticon_path,
+        icns_file,
+        target
+      ])
+    else:
+      if args.verbose:
+        print("[%s] Setting icon for target %s with sips/DeRez/Rez/SetFile." % (print_prefix, target))
+
+      # sips: add an icns resource fork to the icns file
+      subprocess.check_call([
+        sips_path,
+        "-i",
+        icns_file
+      ])
+
+      if original_target:
+        if not os.path.isdir(target):
+          sys.stderr.write("[%s] Warning: the target path does not appear to be a folder. Setting the icon using Rez will probably fail. Try --set-icon-using seticon instead.\n\n" % print_prefix)
+
+        temp_file = os.path.join(temp_folder, "tmpicns.rsrc")
+        target_icon = os.path.join(target, "Icon\r")
+
+        # DeRez: export the icns resource from the icns file
+        with open(temp_file, "w") as f:
+          subprocess.check_call([
+            DeRez_path,
+            "-only",
+            "icns",
+            icns_file
+          ], stdout=f)
+
+        # Rez: add exported icns resource to the resource fork of target/Icon^M
+        try:
+          # If XCode command line tools are not installed, here's where we'd first run into issues.
+          subprocess.check_call([
+            Rez_path,
+            "-append",
+            temp_file,
+            "-o",
+            target_icon
+          ])
+        except OSError as e:
+          # If `Rez` is not installed, we get `FileNotFoundError` in higher versions of Python. But we have to use `OSError` for compatibility with Python 2.7.
+          sys.stderr.write("[%s] Could not set the target icon, probably because Rez is not installed. If you want to use it, make sure XCode command line tools are installed.\n" % print_prefix)
+          sys.stderr.write("%s" % e)
+          sys.exit(1)
+
+        # SetFile: set custom icon attribute
+        subprocess.check_call([
+          SetFile_path,
+          "-a",
+          "C",
+          target
+        ])
+
+        # SetFile: set invisible file attribute
+        subprocess.check_call([
+          SetFile_path,
+          "-a",
+          "V",
+          target_icon
+        ])
 
     # Clean up.
     if not DEBUG:
